# Window-effect.py        I. Marti-Vidal       IMPRS Blackboard Lectures 2011.
import numpy as np
import pylab as pl
# SCRIPT TO COMPUTE THE "WINDOW EFFECT" IN A GALAXY-CORRELATION ANALYSIS
# USING THE MONTE CARLO APPROACH.

# For a given "window" on the sky and a given galaxy inside the window,
# this script computes spheres of different sizes around that galaxy and 
# estimates, for each sphere, the fraction of its surface that falls inside 
# the window.
#
# This is useful when computing statistics related to object clustering
# (i.e., number of objects around a given point as a function of the 
# distance to the point).



#####################################################
# Set the window of our whole galaxy sample:
RA1 = 0.0        # Minimum RA (hr)
RA2 = 3.0        # Maximum RA (hr)
Dec1 = 0.0       # Minimum Dec (deg)
Dec2 = 15.0      # Maximum Dec (deg)
MaxD = 1.0       # Maximum distance (arb. units)
#####################################################
# Set location of a given galaxy (of course, it must be in the window):
GalRA = 2.2      # RA (hr)
GalDec = 12.0    # Dec (deg)
Dist =   0.46    # Distance (arb. units)
#####################################################
# Set Monte Carlo parameters:
nmonte = 50000   # Number of points to perform the integral in the window.
mindist = 0.01   # Minimum radius of the sphere around the galaxy.
maxdist = 0.75   # Maximum radius of the sphere around the galaxy.
numdist = 100.   # Number of spheres to compute.


# SCRIPT STARTS:

# Convert the window edges to radians:
hr2rad = 15.*np.pi/180.
deg2rad = np.pi/180.
[RA1, RA2] = [RA1*hr2rad,RA2*hr2rad]
[Dec1, Dec2] = [Dec1*deg2rad,Dec2*deg2rad]


# Function to compute the celestial coordinates of a point distribution 
# (Monte) around a given galaxy (Gal). All the points are at a given 
# distance (Radi) from the galaxy:

def RADec(Gal, Monte, Radi):

#  Earth-centered Cartesian coordinates of the points:
 Xtot = Gal[0]+Monte[0]*Radi
 Ytot = Gal[1]+Monte[1]*Radi
 Ztot = Gal[2]+Monte[2]*Radi

# Convert to spherical (Earth-centered) coordinates:
 TotDist = np.sqrt(Xtot*Xtot+Ytot*Ytot+Ztot*Ztot)
 Dec = np.arcsin(Ztot/TotDist)
 RA = np.arctan2(Ytot,Xtot)
 return [RA, Dec, TotDist]



# STEP 1: Compute Cartesian coordinates of the galaxy (RA=0 -> X axis)

global GalSph; GalSph = [GalRA*hr2rad,GalDec*deg2rad,Dist]

x = np.cos(GalSph[0])*np.cos(GalSph[1])*GalSph[2]
y = np.sin(GalSph[0])*np.cos(GalSph[1])*GalSph[2]
z = np.sin(GalSph[1])*GalSph[2]

global GalCar; GalCar = [x,y,z]


# STEP 2: Generate points uniformly distributed on the unit sphere
# (i.e., the galaxy is at the center):

Lon = np.random.uniform(0.,2.*np.pi,nmonte)
Lat = np.arcsin(2.*(np.random.uniform(0.,1.,nmonte)-0.5))

# Convert to Cartesian coords. (galaxy is still at the center):
X = np.cos(Lat)*np.cos(Lon)
Y = np.cos(Lat)*np.sin(Lon)
Z = np.sin(Lat)


# STEP 3: Loop over all the spheres to simulate around the galaxy.
#         Note: you can easily adapt this code to work for a set of
#         galaxies, instead of just one.

rs = np.linspace(mindist,maxdist,numdist)
integral = []

for r in rs:

# Compute the celestial coordinates of the points on the sphere:
 RA, Dec, D = RADec(GalCar,[X,Y,Z],r)

# Check how many points fall inside the window:

 SelRA = np.logical_and(RA>RA1,RA<RA2)
 SelDec = np.logical_and(Dec>Dec1,Dec<Dec2)
 SelDist = D<MaxD
 
 SelCoord = np.logical_and(SelRA,SelDec)
 SelTot = np.sum(np.logical_and(SelCoord,SelDist))

# Add result to the final vector:
 integral.append(float(SelTot)/nmonte)

# Plot the results:

pl.plot(rs, integral)
pl.xlabel("Distance from the galaxy (arb. units)")
pl.title("Fraction of the sphere's surface inside the window")
pl.ylabel("Area in window / Total area")
pl.show()


